<?php
namespace MK\Test\Tests\Unit\Controller;

/**
 * Test case.
 */
class RootObjectControllerTest extends \TYPO3\CMS\Core\Tests\UnitTestCase
{
    /**
     * @var \MK\Test\Controller\RootObjectController
     */
    protected $subject = null;

    protected function setUp()
    {
        parent::setUp();
        $this->subject = $this->getMockBuilder(\MK\Test\Controller\RootObjectController::class)
            ->setMethods(['redirect', 'forward', 'addFlashMessage'])
            ->disableOriginalConstructor()
            ->getMock();
    }

    protected function tearDown()
    {
        parent::tearDown();
    }

    /**
     * @test
     */
    public function listActionFetchesAllRootObjectsFromRepositoryAndAssignsThemToView()
    {

        $allRootObjects = $this->getMockBuilder(\TYPO3\CMS\Extbase\Persistence\ObjectStorage::class)
            ->disableOriginalConstructor()
            ->getMock();

        $rootObjectRepository = $this->getMockBuilder(\MK\Test\Domain\Repository\RootObjectRepository::class)
            ->setMethods(['findAll'])
            ->disableOriginalConstructor()
            ->getMock();
        $rootObjectRepository->expects(self::once())->method('findAll')->will(self::returnValue($allRootObjects));
        $this->inject($this->subject, 'rootObjectRepository', $rootObjectRepository);

        $view = $this->getMockBuilder(\TYPO3\CMS\Extbase\Mvc\View\ViewInterface::class)->getMock();
        $view->expects(self::once())->method('assign')->with('rootObjects', $allRootObjects);
        $this->inject($this->subject, 'view', $view);

        $this->subject->listAction();
    }

    /**
     * @test
     */
    public function showActionAssignsTheGivenRootObjectToView()
    {
        $rootObject = new \MK\Test\Domain\Model\RootObject();

        $view = $this->getMockBuilder(\TYPO3\CMS\Extbase\Mvc\View\ViewInterface::class)->getMock();
        $this->inject($this->subject, 'view', $view);
        $view->expects(self::once())->method('assign')->with('rootObject', $rootObject);

        $this->subject->showAction($rootObject);
    }

    /**
     * @test
     */
    public function createActionAddsTheGivenRootObjectToRootObjectRepository()
    {
        $rootObject = new \MK\Test\Domain\Model\RootObject();

        $rootObjectRepository = $this->getMockBuilder(\MK\Test\Domain\Repository\RootObjectRepository::class)
            ->setMethods(['add'])
            ->disableOriginalConstructor()
            ->getMock();

        $rootObjectRepository->expects(self::once())->method('add')->with($rootObject);
        $this->inject($this->subject, 'rootObjectRepository', $rootObjectRepository);

        $this->subject->createAction($rootObject);
    }

    /**
     * @test
     */
    public function editActionAssignsTheGivenRootObjectToView()
    {
        $rootObject = new \MK\Test\Domain\Model\RootObject();

        $view = $this->getMockBuilder(\TYPO3\CMS\Extbase\Mvc\View\ViewInterface::class)->getMock();
        $this->inject($this->subject, 'view', $view);
        $view->expects(self::once())->method('assign')->with('rootObject', $rootObject);

        $this->subject->editAction($rootObject);
    }

    /**
     * @test
     */
    public function updateActionUpdatesTheGivenRootObjectInRootObjectRepository()
    {
        $rootObject = new \MK\Test\Domain\Model\RootObject();

        $rootObjectRepository = $this->getMockBuilder(\MK\Test\Domain\Repository\RootObjectRepository::class)
            ->setMethods(['update'])
            ->disableOriginalConstructor()
            ->getMock();

        $rootObjectRepository->expects(self::once())->method('update')->with($rootObject);
        $this->inject($this->subject, 'rootObjectRepository', $rootObjectRepository);

        $this->subject->updateAction($rootObject);
    }
}
