<?php
namespace MK\Test\Tests\Unit\Domain\Model;

/**
 * Test case.
 */
class RootObjectTest extends \TYPO3\CMS\Core\Tests\UnitTestCase
{
    /**
     * @var \MK\Test\Domain\Model\RootObject
     */
    protected $subject = null;

    protected function setUp()
    {
        parent::setUp();
        $this->subject = new \MK\Test\Domain\Model\RootObject();
    }

    protected function tearDown()
    {
        parent::tearDown();
    }

    /**
     * @test
     */
    public function getTitleReturnsInitialValueForString()
    {
        self::assertSame(
            '',
            $this->subject->getTitle()
        );
    }

    /**
     * @test
     */
    public function setTitleForStringSetsTitle()
    {
        $this->subject->setTitle('Conceived at T3CON10');

        self::assertAttributeEquals(
            'Conceived at T3CON10',
            'title',
            $this->subject
        );
    }

    /**
     * @test
     */
    public function getChildrenReturnsInitialValueForChild()
    {
        $newObjectStorage = new \TYPO3\CMS\Extbase\Persistence\ObjectStorage();
        self::assertEquals(
            $newObjectStorage,
            $this->subject->getChildren()
        );
    }

    /**
     * @test
     */
    public function setChildrenForObjectStorageContainingChildSetsChildren()
    {
        $child = new \MK\Test\Domain\Model\Child();
        $objectStorageHoldingExactlyOneChildren = new \TYPO3\CMS\Extbase\Persistence\ObjectStorage();
        $objectStorageHoldingExactlyOneChildren->attach($child);
        $this->subject->setChildren($objectStorageHoldingExactlyOneChildren);

        self::assertAttributeEquals(
            $objectStorageHoldingExactlyOneChildren,
            'children',
            $this->subject
        );
    }

    /**
     * @test
     */
    public function addChildToObjectStorageHoldingChildren()
    {
        $child = new \MK\Test\Domain\Model\Child();
        $childrenObjectStorageMock = $this->getMockBuilder(\TYPO3\CMS\Extbase\Persistence\ObjectStorage::class)
            ->setMethods(['attach'])
            ->disableOriginalConstructor()
            ->getMock();

        $childrenObjectStorageMock->expects(self::once())->method('attach')->with(self::equalTo($child));
        $this->inject($this->subject, 'children', $childrenObjectStorageMock);

        $this->subject->addChild($child);
    }

    /**
     * @test
     */
    public function removeChildFromObjectStorageHoldingChildren()
    {
        $child = new \MK\Test\Domain\Model\Child();
        $childrenObjectStorageMock = $this->getMockBuilder(\TYPO3\CMS\Extbase\Persistence\ObjectStorage::class)
            ->setMethods(['detach'])
            ->disableOriginalConstructor()
            ->getMock();

        $childrenObjectStorageMock->expects(self::once())->method('detach')->with(self::equalTo($child));
        $this->inject($this->subject, 'children', $childrenObjectStorageMock);

        $this->subject->removeChild($child);
    }
}
